@@ -6,10 +6,12 @@ import datetime
 
 # Backwards compatibility with AuthKit
 from usermanager.driver.base import *
+from pipestack.ensure import ensure_function_bag
 
 def _nothing(a):
     return a
 
+@ensure_function_bag('user')
 def create_tables(service, drop_first=False, name='user'):
     database_name = service[name].connection_name
     conn = service[database_name].connect()
@@ -29,7 +31,12 @@ def create_tables(service, drop_first=False, name='user'):
             uid serial UNIQUE NOT NULL,
             username character varying(255) UNIQUE NOT NULL,
             password character varying(255) NOT NULL,
-            group_uid integer REFERENCES authkit_group (uid)
+            group_uid integer REFERENCES authkit_group (uid),
+            terms boolean NOT NULL DEFAULT FALSE,
+            email character varying(255) NOT NULL
+            --login_count integer NOT NULL DEFAULT 0,
+            --last_login_time timestamp  NULL default NOW(),
+            --last_login_ip varchar(15) NOT NULL
         );
 
         CREATE TABLE authkit_user_role (
@@ -38,17 +45,85 @@ def create_tables(service, drop_first=False, name='user'):
             role_uid integer REFERENCES authkit_role (uid)
         );
 
+        CREATE TABLE authkit_login (
+            uid serial UNIQUE NOT NULL,
+            user_username character varying(255) REFERENCES authkit_user (username),
+            login_time timestamp NOT NULL default NOW(),
+            login_ip varchar(15) NOT NULL
+        );
     """
     if drop_first:
         sql += """
-            DROP TABLE authkit_group;
-            DROP TABLE authkit_role;
-            DROP TABLE authkit_user;
-            DROP TABLE authkit_user_role;
+            DROP TABLE IF EXISTS authkit_login;
+            DROP TABLE IF EXISTS authkit_user_role;
+            DROP TABLE IF EXISTS authkit_role;
+            DROP TABLE IF EXISTS authkit_user;
+            DROP TABLE IF EXISTS authkit_group;
         """
     cursor.execute(sql)
     cursor.close()
 
+import datetime
+@ensure_function_bag('user', 'database')
+def logged_in(bag, username, ip):
+    bag.database.insert_record(
+        "authkit_login",
+        dict(login_ip=ip, user_username=username),
+        "uid",
+    )
+
+@ensure_function_bag('user')
+def drop_tables(service, name='user'):
+    database_name = service[name].connection_name
+    conn = service[database_name].connect()
+    cursor = conn.cursor()
+    sql = """
+        DROP TABLE IF EXISTS authkit_login;
+        DROP TABLE IF EXISTS authkit_user_role;
+        DROP TABLE IF EXISTS authkit_role;
+        DROP TABLE IF EXISTS authkit_user;
+        DROP TABLE IF EXISTS authkit_group;
+    """
+    cursor.execute(sql)
+    cursor.close()
+
+from conversionkit import Conversion, chainConverters
+def validPassword(length=6):
+    import string
+    def validPassword_converter(conversion, bag):
+        if len(conversion.value) < length:
+            conversion.error = 'Passwords must be at least %s characters long'%length
+            return
+        for char in conversion.value:
+            if char not in string.printable:
+                conversion.error = 'Passwords may not contain the %r character'%(char.encode('utf8'),)
+                return
+        conversion.result = conversion.value
+    return validPassword_converter
+
+def validUsername(length=6):
+    import string
+    def validUsername_converter(conversion, bag):
+        if len(conversion.value) < length:
+            conversion.error = 'Usernames must be at least %s characters long'%length
+            return
+        for char in conversion.value:
+            if char not in string.digits+'_'+string.ascii_letters:
+                conversion.error = 'Usernames may contain only letters, numbers and underscores, not the %r character'%(char.encode('utf-8'),)
+                return
+        conversion.result = conversion.value
+    return validUsername_converter
+
+def userExists():
+    def userExists_converter(conversion, bag):
+        if user_exists(bag, conversion.value):
+            conversion.error = 'User %r already exists'%(conversion.value.encode('utf-8'))
+        else:
+            conversion.result = conversion.value
+    return userExists_converter
+
+valid_new_username = chainConverters(validUsername(), userExists())
+
 # Create Methods
 def user_create(
     service,
@@ -57,18 +132,32 @@ def user_create(
     group=None,
     name='user',
     encrypt=_nothing,
+    terms=False,
+    email='',
 ):
     """\
     Create a new user with the username, password and group name specified.
     """
-    if ' ' in username:
-        raise UserManagerError("Usernames cannot contain space characters")
-    if user_exists(service, username, name=name):
-        raise UserManagerError("User %r already exists"%username) 
+    conversion = Conversion(username).perform(valid_new_username, service)
+    if not conversion.successful:
+        raise Exception(conversion.error)
+    username = conversion.result
     if group is not None and not group_exists(service, group, name=name):
         raise NoSuchGroupError(
             "There is no such group %r"%group
         )
+    return user_create_store(service, username, password, group, name, encrypt, terms, email)
+
+def user_create_store(
+    service,
+    username,
+    password,
+    group=None,
+    name='user',
+    encrypt=_nothing,
+    terms=False,
+    email='',
+):
     database_name = service[name].connection_name
     conn = service[database_name].connect()
     cursor = conn.cursor()
@@ -84,9 +173,9 @@ def user_create(
         group_uid = None
     cursor.execute(
         """
-        INSERT INTO authkit_user (username, password, group_uid) VALUES (%s, %s, %s)
+        INSERT INTO authkit_user (username, password, group_uid, terms, email) VALUES (%s, %s, %s, %s, %s)
         """,
-        (username, encrypt(password), group_uid)
+        (username, encrypt(password), group_uid, terms, email)
     )
     cursor.close()
 
@@ -211,6 +300,7 @@ def group_delete(service, group, name='user'):
         cursor.close()
 
 # Existence Methods
+@ensure_function_bag('user')
 def user_exists(service, username, name='user'):
     """\
     Returns ``True`` if a user exists with the given username, ``False``
