@@ -1,7 +1,8 @@
 import ldap
 from pipestack.pipe import Marble, MarblePipe
 from pipestack.ensure import ensure_method_bag, ensure_self_marble
-from conversionkit import Field
+from conversionkit import Field, toDictionary
+from recordconvert import toListOfRecords
 from stringconvert import unicodeToUnicode, unicodeToBoolean
 from databasepipe.setup import BaseSetupCmd
 import logging
@@ -21,6 +22,21 @@ def listOfUsernames(split_name=False, extract_organisation=False):
             )
     return listOfUsernames_converter
 
+bypass_user = toListOfRecords(
+     toDictionary(
+         converters = {
+             'username': Field(
+                unicodeToUnicode(), 
+                missing_or_empty_default=None,
+             ),
+             'password': Field(
+                unicodeToUnicode(), 
+                missing_or_empty_default=None,
+             ),
+         }
+    )
+)
+
 class SetupCmd(BaseSetupCmd):
 
     table_names_string = 'authkit_login'
@@ -66,13 +82,10 @@ class LdapUserMarble(Marble):
             )
     
     def user_has_password(marble, username, password):
-        if marble.config.bypass_username is not None and \
-           marble.config.bypass_password is not None and \
-           username == marble.config.bypass_username:
-            if password == marble.config.bypass_password:
-                return True
-            else:
-                return False
+        if marble.config.bypass_user:
+            for user in marble.config.bypass_user:
+                if username == user.username and password == user.password:
+                    return True
         if marble.config.restrict_ldap_usernames and \
            username not in marble.config.restrict_ldap_usernames:
             log.debug(
@@ -115,18 +128,11 @@ class LdapUserPipe(MarblePipe):
             ),
             missing_default=False,
         ),
-        'bypass_username': Field(
-            unicodeToUnicode(), 
-            missing_or_empty_default=None,
-        ),
+        'bypass_user': bypass_user,
         'restrict_ldap_usernames': Field(
             listOfUsernames(), 
             missing_or_empty_default=None,
         ),
-        'bypass_password': Field(
-            unicodeToUnicode(), 
-            missing_or_empty_default=None,
-        ),
         'log_logins': Field(
             unicodeToBoolean(), 
             missing_or_empty_default=None,
